@@ -1,4 +1,4 @@
-// Copyright 2012 The Go Authors.  All rights reserved.
+// Copyright 2012 The Go Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style
 // license that can be found in the LICENSE file.
 
@@ -10,7 +10,7 @@ import "testing"
 // as expected.
 func TestRead(t *testing.T) {
 	f := Fopen("file_test.go", "r")
-	if f == nil {
+	if f.Swigcptr() == 0 {
 		t.Fatal("fopen failed")
 	}
 	if Fgetc(f) != '/' || Fgetc(f) != '/' || Fgetc(f) != ' ' || Fgetc(f) != 'C' {
@@ -20,3 +20,9 @@ func TestRead(t *testing.T) {
 		t.Error("fclose failed")
 	}
 }
+
+func TestF(t *testing.T) {
+	if x := F(); x != 1 {
+		t.Fatalf("x = %d, want 1", x)
+	}
+}
